@@ -310,12 +310,13 @@ file_notifier_traverse_tree_foreach (GFile    *file,
 		 * to crawl (i.e. embedded root directories, that would
 		 * be processed when that root is being crawled).
 		 */
-		if (!tracker_indexing_tree_file_is_root (priv->indexing_tree, file)) {
+		if (file != priv->current_index_root->root &&
+		    !tracker_indexing_tree_file_is_root (priv->indexing_tree, file)) {
 			gchar *uri;
 
 			uri = g_file_get_uri (file);
-			g_critical ("File '%s' has no disk nor store mtime",
-			            uri);
+			g_debug ("File '%s' has no disk nor store mtime",
+			         uri);
 			g_free (uri);
 		}
 	}
@@ -622,7 +623,10 @@ crawl_directory_in_current_root (TrackerFileNotifier *notifier)
 		return FALSE;
 
 	priv->current_index_root->current_dir = directory;
-	g_cancellable_reset (priv->cancellable);
+
+	if (priv->cancellable)
+		g_object_unref (priv->cancellable);
+	priv->cancellable = g_cancellable_new ();
 
 	if ((priv->current_index_root->flags & TRACKER_DIRECTORY_FLAG_RECURSE) == 0) {
 		/* Don't recurse */
@@ -653,11 +657,14 @@ finish_current_directory (TrackerFileNotifier *notifier,
 	directory = priv->current_index_root->current_dir;
 	priv->current_index_root->current_dir = NULL;
 
-	/* We dispose regular files here, only directories are cached once crawling
-	 * has completed.
+	/* If crawling was interrupted, we take all collected info as invalid.
+	 * Otherwise we dispose regular files here, only directories are
+	 * cached once crawling has completed.
 	 */
 	tracker_file_system_forget_files (priv->file_system,
 	                                  directory,
+					  interrupted ?
+					  G_FILE_TYPE_UNKNOWN :
 	                                  G_FILE_TYPE_REGULAR);
 
 	if (interrupted || !crawl_directory_in_current_root (notifier)) {
@@ -690,6 +697,54 @@ finish_current_directory (TrackerFileNotifier *notifier,
 	g_object_unref (directory);
 }
 
+static gboolean
+root_data_remove_directory (RootData *data,
+			    GFile    *directory)
+{
+	GList *l = data->pending_dirs->head, *next;
+	GFile *file;
+
+	while (l) {
+		file = l->data;
+		next = l->next;
+
+		if (g_file_equal (file, directory) ||
+		    g_file_has_prefix (file, directory)) {
+			g_queue_remove (data->pending_dirs, file);
+			g_object_unref (file);
+		}
+
+		l = next;
+	}
+
+	return (g_file_equal (data->current_dir, directory) ||
+		g_file_has_prefix (data->current_dir, directory));
+}
+
+static void
+file_notifier_current_root_check_remove_directory (TrackerFileNotifier *notifier,
+						   GFile               *file)
+{
+	TrackerFileNotifierPrivate *priv;
+
+	priv = notifier->priv;
+
+	if (priv->current_index_root &&
+	    root_data_remove_directory (priv->current_index_root, file)) {
+		g_cancellable_cancel (priv->cancellable);
+		tracker_crawler_stop (priv->crawler);
+
+		if (!crawl_directory_in_current_root (notifier)) {
+			if (priv->current_index_root) {
+				root_data_free (priv->current_index_root);
+				priv->current_index_root = NULL;
+			}
+
+			notifier_check_next_root (notifier);
+		}
+	}
+}
+
 /* Query for directory contents, used to look for deleted contents in those */
 static void
 sparql_contents_query_cb (GObject      *object,
@@ -1032,6 +1087,7 @@ monitor_item_created_cb (TrackerMonitor *monitor,
 				 * filter, remove parent directory altogether
 				 */
 				g_signal_emit (notifier, signals[FILE_DELETED], 0, parent);
+				file_notifier_current_root_check_remove_directory (notifier, parent);
 				g_object_unref (parent);
 				return;
 			}
@@ -1205,6 +1261,8 @@ monitor_item_deleted_cb (TrackerMonitor *monitor,
 	g_object_ref (canonical);
 	g_signal_emit (notifier, signals[FILE_DELETED], 0, canonical);
 
+	file_notifier_current_root_check_remove_directory (notifier, canonical);
+
 	/* Remove the file from the cache (works recursively for directories) */
 	tracker_file_system_forget_files (priv->file_system,
 	                                  canonical,
@@ -1302,6 +1360,7 @@ monitor_item_moved_cb (TrackerMonitor *monitor,
 			}
 
 			g_signal_emit (notifier, signals[FILE_DELETED], 0, file);
+			file_notifier_current_root_check_remove_directory (notifier, file);
 		} else {
 			/* Handle move */
 			if (is_directory) {
@@ -1460,12 +1519,14 @@ tracker_file_notifier_finalize (GObject *object)
 		g_object_unref (priv->data_provider);
 	}
 
-	g_cancellable_cancel (priv->cancellable);
+	if (priv->cancellable) {
+		g_cancellable_cancel (priv->cancellable);
+		g_object_unref (priv->cancellable);
+	}
 
 	g_object_unref (priv->crawler);
 	g_object_unref (priv->monitor);
 	g_object_unref (priv->file_system);
-	g_object_unref (priv->cancellable);
 	g_clear_object (&priv->connection);
 
 	if (priv->current_index_root)
@@ -1651,7 +1712,6 @@ tracker_file_notifier_init (TrackerFileNotifier *notifier)
 		                             TrackerFileNotifierPrivate);
 
 	priv->connection = tracker_sparql_connection_get (NULL, &error);
-	priv->cancellable = g_cancellable_new ();
 
 	if (error) {
 		g_warning ("Could not get SPARQL connection: %s\n",
